import bpy, mathutils
import os
from math import radians
import sys
import xml.etree.ElementTree as ET

# read arguments given on the command line
sys_argv = sys.argv
sys_argv = sys_argv[sys_argv.index("--") + 1:]  
temp_directory = sys_argv[0]


def look_at(obj_camera, point):
    loc_camera = obj_camera.matrix_world.to_translation()
    direction = point - loc_camera
    rot_quat = direction.to_track_quat('-Z', 'Y') # point camera -Z axis (Y-vec up)
    obj_camera.rotation_euler = rot_quat.to_euler()


def createLamp(lampname, location, lamptype, object_data):
    scene = bpy.context.scene
    lamp_data = bpy.data.lamps.new(name=lampname, type=lamptype)
    lamp_data.use_nodes = True
    rgb = object_data[0]
    strength = object_data[1]
    if lamptype == 'AREA':
        lamp_data.shape = 'RECTANGLE'
        lamp_data.size = object_data[2]
        lamp_data.size_y = object_data[3]
        light_falloff = object_data[4]
    elif lamptype in ['SPOT']:
        light_falloff = object_data[4]
        size = object_data[2]
    elif lamptype in ['POINT']:
        light_falloff = object_data[3]
        size = object_data[2]
    else: #SUN
        size = object_data[2]
    rgb = [x/255.0 for x in rgb]
    rgb.append(1) # append to list of 4 items
    if lamptype != 'AREA':
        if bpy.context.scene.render.use_freestyle:
            lamp_data.shadow_soft_size = 0.001 # force hard shadows in to get cartoonish look
        else:
            lamp_data.shadow_soft_size = size
    if lamptype == 'SPOT':
        lamp_data.spot_size = radians(object_data[3])
    lamp_data.node_tree.nodes["Emission"].inputs[0].default_value = rgb # color
    if lamptype == 'SUN':
        # sun lamp doesn't need falloff node
        lamp_data.node_tree.nodes["Emission"].inputs[1].default_value = strength # strength
    else:
        falloffNode = lamp_data.node_tree.nodes.new(type="ShaderNodeLightFalloff")
        falloffNode.location.x = -200
        falloffNode.location.y = 300
        falloffNode.inputs[0].default_value = strength
        falloffNode.inputs[1].default_value = 0.9 # smoothing reduces brightest spot => less noise
        links = lamp_data.node_tree.links
        if light_falloff == 'Linear':
            # link nodes
            link = links.new(falloffNode.outputs[1], lamp_data.node_tree.nodes["Emission"].inputs[1])
        elif light_falloff == 'Constant':
            link = links.new(falloffNode.outputs[2], lamp_data.node_tree.nodes["Emission"].inputs[1])
        elif light_falloff == 'Quadratic':
            link = links.new(falloffNode.outputs[0], lamp_data.node_tree.nodes["Emission"].inputs[1])
    lamp_object = bpy.data.objects.new(name=lampname, object_data=lamp_data)
    scene.objects.link(lamp_object)
    lamp_object.location = location[0:3]
    point = mathutils.Vector((location[3], location[4], location[5]))
    look_at(lamp_object, point)
    return lamp_object



def do_import_vc():
    uri_root = temp_directory
    # read scene structure from .vcbx file
    # - lights, camera, nodes, 
    rec_file_uri = os.path.join(uri_root, 'recording.vcbx' ) 
    rec_file = open(rec_file_uri, 'r')
    mydata = rec_file.read()
    rec_file.close()
    rec_data = eval(mydata) #convert string to list (data is saved as data=str(list) so it can be evalued back to list)
    config_file_uri = os.path.join(uri_root, 'render_config.xml' )
    tree = ET.parse(config_file_uri)
    root = tree.getroot()
    config_dict = {}
    for kid in root:
        config_dict[kid.tag] = kid.text
    materials = {}
    # set render things before starting to import stuff
    bpy.context.scene.render.engine = config_dict['engine']
    openinblender = config_dict['openinblender'] == 'True'
    bpy.context.scene.render.resolution_x = int(config_dict['resolution_x'])
    bpy.context.scene.render.resolution_y = int(config_dict['resolution_y'])
    bpy.context.scene.render.resolution_percentage = int(config_dict['resolution_percentage'])
    bpy.context.scene.render.use_border = False
    bpy.context.scene.cycles.sample_clamp_direct = float(config_dict['sample_clamp_direct'])
    bpy.context.scene.cycles.sample_clamp_indirect = float(config_dict['sample_clamp_indirect'])
    bpy.context.scene.cycles.transparent_max_bounces = int(config_dict['transparent_max_bounces'])
    bpy.context.scene.cycles.transparent_min_bounces = int(config_dict['transparent_min_bounces'])
    bpy.context.scene.cycles.max_bounces = int(config_dict['max_bounces'])
    bpy.context.scene.cycles.min_bounces = int(config_dict['min_bounces'])
    bpy.context.scene.cycles.diffuse_bounces = int(config_dict['diffuse_bounces'])
    bpy.context.scene.cycles.glossy_bounces = int(config_dict['glossy_bounces'])
    bpy.context.scene.cycles.transmission_bounces = int(config_dict['transmission_bounces'])
    aobounces = int(config_dict['aobounces'])
    if aobounces:
        bpy.context.scene.render.use_simplify = True
        bpy.context.scene.cycles.ao_bounces_render = aobounces
    bpy.context.scene.cycles.volume_bounces = int(config_dict['volume_bounces'])
    bpy.context.scene.cycles.caustics_reflective = config_dict['caustics_reflective'] == 'True'
    bpy.context.scene.cycles.caustics_refractive = config_dict['caustics_refractive'] == 'True'
    denoising = float(config_dict['denoising'])
    if denoising > 0:
        #bpy.context.scene.use_denoising = True
        bpy.context.scene.render.layers[0].cycles.use_denoising = True
        #bpy.context.scene.denoising_strength = min(denoising, 1.0) # clamp between 0...1
        bpy.context.scene.render.layers[0].cycles.denoising_strength = min(denoising, 1.0) # clamp between 0...1
    bpy.context.scene.render.use_motion_blur = False
    bpy.context.scene.render.tile_x = int(config_dict['tile_x'])
    bpy.context.scene.render.tile_y = int(config_dict['tile_y'])
    bpy.context.scene.cycles.samples = int(config_dict['samples'])
    bpy.context.scene.frame_start = frameS = int(config_dict['frame_start'])
    bpy.context.scene.frame_end = frameE = int(config_dict['frame_end'])
    bpy.context.scene.render.filepath = config_dict['filepath']
    bpy.context.scene.cycles.device = config_dict['device']
    bpy.context.scene.render.use_freestyle = config_dict['use_freestyle'] == 'True'
    if bpy.context.scene.render.use_freestyle:
        #bpy.context.space_data.context = 'RENDER'
        bpy.context.scene.cycles.pixel_filter_type = 'BOX'
        bpy.context.scene.cycles.debug_bvh_type = 'STATIC_BVH'
        bpy.context.scene.render.use_persistent_data = True
        bpy.context.scene.cycles.debug_use_spatial_splits = True
        bpy.context.scene.cycles.debug_use_hair_bvh = False
        bpy.context.scene.world.light_settings.use_ambient_occlusion = True
        bpy.context.scene.world.light_settings.ao_factor = 0.1
        bpy.context.scene.world.cycles_visibility.diffuse = False
        bpy.context.scene.world.cycles_visibility.glossy = False
        bpy.context.scene.world.cycles_visibility.scatter = False
        bpy.context.scene.world.cycles_visibility.transmission = False
        bpy.context.scene.cycles.transparent_max_bounces = 0
        bpy.context.scene.cycles.transparent_min_bounces = 0
        bpy.context.scene.cycles.blur_glossy = 0
        bpy.context.scene.cycles.max_bounces = 0
        bpy.context.scene.cycles.min_bounces = 0
        bpy.context.scene.cycles.diffuse_bounces = 0
        bpy.context.scene.cycles.glossy_bounces = 0
        bpy.context.scene.cycles.transmission_bounces = 0
        bpy.context.scene.cycles.volume_bounces = 0
        bpy.context.scene.cycles.transparent_max_bounces = 0
        bpy.data.linestyles["LineStyle"].thickness = 2
        bpy.context.scene.cycles.film_exposure = 3


    bpy.context.scene.render.line_thickness_mode = 'RELATIVE'
    #bpy.context.scene.view_settings.view_transform = 'Filmic'

    depthOFnode = config_dict['depthOFnode']
    depthOfFieldObject = None
    depthOFradius = float(config_dict['depthOFradius'])
    volumescatter = float(config_dict['volumescatter'])

    #
    envimage = config_dict['envimage'].replace('file:///' , '')
    envstrength = float(config_dict['envstrength'])
    envtype = config_dict['envtype']
    envcolorgb = [float(x) for x in config_dict['envcolor'].split(';')]
    envvisible = config_dict['envvisible']
    #
    floorvisible = config_dict['floorvisible']
    floorsize = config_dict['floorsize']
    floortype = config_dict['floortype']
    floorcolorimage = config_dict['floorcolorimage'].replace('file:///' , '')
    floorbumpimage = config_dict['floorbumpimage'].replace('file:///' , '')
    floortexturesize = float(config_dict['floortexturesize'])
    floorcolor = config_dict['floorcolor']
    floor_twoD_bound = eval(config_dict['floor_twoD_bound'])
    floor_corners = floor_twoD_bound # [-X, +X, -Y, +Y]

    CameraFOV = float(config_dict['CameraFOV'])

    # CLEAR scene
    for o in bpy.data.objects:
        if o.type in ['MESH', 'EMPTY', 'CAMERA', 'LAMP']:
            o.select = True
        else:
            o.select = False
    bpy.ops.object.delete()
    for mat in bpy.data.materials:
        bpy.data.materials.remove(mat)
    #
    # Set HDRI (env map)
    #
    scn = bpy.context.scene
    scn.world.use_nodes = True
    wd = scn.world
    nt = bpy.data.worlds[wd.name].node_tree
    if envtype == 'Image':
        envNode = nt.nodes.new(type="ShaderNodeTexEnvironment")
        envTexture_img = bpy.data.images.load(envimage)
        envNode.image = envTexture_img
        backNode = nt.nodes['Background']
        envNode.location.x = backNode.location.x-300
        envNode.location.y = backNode.location.y
        envNodeOut = envNode.outputs['Color']
        backColIn = backNode.inputs['Color']
        backNode.inputs[1].default_value = envstrength
        nt.links.new(envNodeOut, backColIn)
    else: #RGB
        backNode = nt.nodes['Background']
        backColIn = backNode.inputs['Color']
        if len(envcolorgb) == 3:
            ccolor = [0,0,0,1]
            ccolor[:3] = envcolorgb
            envcolorgb = tuple(ccolor)
        envcolorgb = [x**2.2 for x in envcolorgb] # gamma correction
        backColIn.default_value = envcolorgb
        backNode.inputs[1].default_value = envstrength
    bpy.context.scene.world.cycles_visibility.camera = True if envvisible  == 'True' else False
    #
    # Mist
    #
    if volumescatter:
        volumescatterNode = nt.nodes.new(type="ShaderNodeVolumeScatter")
        volumescatterNode.inputs[1].default_value = volumescatter
        volumescatterNode.location.x = backNode.location.x-300
        volumescatterNode.location.y = backNode.location.y-300
        worldoutputNode = nt.nodes['World Output']
        nt.links.new(volumescatterNode.outputs['Volume'], worldoutputNode.inputs['Volume'])
    #
    # Start importing scene
    #
    mycounter = 0
    allcount = len(rec_data)
    matData = loadMAT(os.path.join(uri_root,'VC_to_Blender_materials.mtl'))
    # LOAD MATERIALS BEFORE CREATING THE OBJECTS
    blender_materials = {}
    for materialname in matData:
            mObj = matData[materialname]
            mat = bpy.data.materials.new(name=materialname)
            setupShaderMaterial(mat,
                                mObj.Color,
                                mObj.ColorMap,
                                mObj.Metallic,
                                mObj.Roughness,
                                mObj.NormalMap,
                                mObj.Bumpiness,
                                mObj.Clearcoat,
                                mObj.ClearcoatRougness,
                                mObj.Opacity,
                                #mObj.ProceduralDisplacement,
                                #mObj.DisplacementValue,
                                #mObj.DisplacementScale,
                                mObj.CustomData)
            blender_materials[materialname] = mat
    #
    # Loop rec data to create objects and setting the keyframe data
    #
    for item in rec_data:
        mycounter += 1 
        name = item[0]
        item_type = item[1]
        object_data = item[2]
        position_data = item[3]
        inipos = position_data[0]
        print('Generating %i/%i: "%s"' % (mycounter, allcount, name) )
        if item_type == 'CAMERA':
            new_camera= bpy.ops.object.add(type='CAMERA', location=inipos[0:3])
            camera_object = bpy.context.selected_objects[0]
            camera_object.name = name
            point = mathutils.Vector((inipos[3], inipos[4], inipos[5]))
            look_at(camera_object, point)
            #camera_object.rotation_euler = [0,0,0]
            camera_object.data.clip_end = 5000
            obj_object = camera_object
            camera_object.data.lens_unit = 'FOV'
            camera_object.data.angle = radians(CameraFOV)
            bpy.context.scene.camera = camera_object
        elif item_type in 'POINT_LIGHT':
            lamp_object = createLamp(name, inipos, 'POINT', object_data)
            obj_object = lamp_object
        elif item_type in 'AREA_LIGHT':
            lamp_object = createLamp(name, inipos, 'AREA', object_data)
            obj_object = lamp_object
        elif item_type in 'SUN_LIGHT':
            lamp_object = createLamp(name, inipos, 'SUN', object_data)
            obj_object = lamp_object
        elif item_type in 'SPOT_LIGHT':
            lamp_object = createLamp(name, inipos, 'SPOT', object_data)
            obj_object = lamp_object
        elif item_type == 'NODE':
            uri = os.path.join(uri_root, name)
            uri += '.obj'
            #USE THE VC's blender OBJ IMPORTER (def loadModel) INCLUDED IN THIS FILE
            new_empty = loadmodel(uri, name, blender_materials)
            new_empty.location = inipos[0:3]
            new_empty.rotation_euler = inipos[3:6]
            new_empty.scale = inipos[6:]
            obj_object = new_empty
            if depthOFnode and depthOFnode == name:
                depthOfFieldObject = new_empty
                
        else:
            continue
        #bpy.context.scene.update()
        t=0
        for i,L in enumerate(position_data):
            obj_object.location = L[0], L[1], L[2]
            bpy.context.scene.update()
            if item_type in ['CAMERA', 'AREA_LIGHT', 'SPOT_LIGHT', 'SUN_LIGHT']:
                point = mathutils.Vector((L[3], L[4], L[5]))
                look_at(obj_object, point)
            else:
                obj_object.rotation_euler = L[3], L[4], L[5]
            if item_type == 'NODE':
                obj_object.scale = L[6], L[7], L[8]
            obj_object.keyframe_insert( data_path = 'location', frame = t )
            obj_object.keyframe_insert( data_path = 'rotation_euler', frame = t )
            obj_object.keyframe_insert( data_path = 'scale', frame = t )
            t+=1
    #
    # Depth Of Field
    #
    if depthOfFieldObject:
        # depthOfFieldObject was cached during loop if defined
        camera_object.data.dof_object = depthOfFieldObject
        camera_object.data.cycles.aperture_size = depthOFradius
    #
    # Floor
    #
    if floorvisible == 'True':
        Lx = floor_corners[1]-floor_corners[0]
        Ly = floor_corners[3]-floor_corners[2]
        MidX = (floor_corners[1]+floor_corners[0])/2.0
        MidY = (floor_corners[2]+floor_corners[3])/2.0
        #floorsize
        plane = bpy.ops.mesh.primitive_plane_add(location=(MidX,MidY,-0.001), radius = .5)
        plane_object = bpy.context.selected_objects[0]
        bpy.ops.transform.resize(value=(Lx, Ly, 1.0))
        bpy.ops.object.editmode_toggle()
        bpy.ops.uv.unwrap(method='ANGLE_BASED', margin=0.001)
        bpy.ops.object.editmode_toggle()
        
        scale = floortexturesize
        #uvs = [(0,0),(2,0),(2,2),(0,2)]
        uvs = [(0,0),(Lx/scale,0),(Lx/scale,Ly/scale),(0,Ly/scale)]
        for loop, coords in zip(plane_object.data.loops, uvs):
            uv_coords = plane_object.data.uv_layers.active.data[loop.index].uv
            uv_coords[0] = coords[0]
            uv_coords[1] = coords[1]
        ob = bpy.context.active_object
        mat = bpy.data.materials.new(name="FLOORMATERIAL_BlenderToVC_X")
        ob.data.materials.append(mat)
        floortype
        metallic = 0.0
        roughness = 0.9
        clearcoat = 0.4
        clearcoat_rouhness = 0.6
        bumpiness = 50
        #procedural_displacement = ''
        custom_data = ''

        node_noise,node_math = None, None
        mat.use_nodes = True
        nodes = mat.node_tree.nodes
        for node in nodes:
            nodes.remove(node)
        # create principled
        node_principled = nodes.new(type="ShaderNodeBsdfPrincipled")
        node_principled.location = 0,0
        node_principled.inputs[4].default_value = 1.0 if metallic > 0 else 0.0 #material is metal or not.. nothing between
        node_principled.inputs[7].default_value = roughness
        node_principled.inputs[12].default_value = clearcoat
        node_principled.inputs[13].default_value = clearcoat_rouhness
        # create output node
        node_output = nodes.new(type='ShaderNodeOutputMaterial')   
        node_output.location = 300,0
        # create texture     node
        if floorcolorimage:
            node_texture = nodes.new(type='ShaderNodeTexImage')
            node_texture.location = -600,100
            texture_img = bpy.data.images.load(floorcolorimage)
            node_texture.image = texture_img
        else:
            # text color
            if len(col) == 3:
                ccolor = [0,0,0,1]
                ccolor[:3] = [float(x) for x in floorcolor.split(';')]
                col = tuple(ccolor)
            col = [x**2.2 for x in col] # gamma correction
            node_principled.inputs[0].default_value = col
        if floorbumpimage and not bpy.context.scene.render.use_freestyle:
            node_normalmap = nodes.new(type='ShaderNodeTexImage')
            node_normalmap.color_space = 'NONE' #non color data
            node_normalmap.location = -600,-300
            normalmap_img = bpy.data.images.load(floorbumpimage)
            node_normalmap.image = normalmap_img
            #
            node_normalvector = nodes.new(type="ShaderNodeNormalMap")
            node_normalvector.location = -300,-300
            node_normalvector.inputs[0].default_value = bumpiness*2
        if custom_data.startswith('NOISE'):
            try:
                c_type, displacement_scale, displacement_value = custom_data.split('/')
                #displacement_value, displacement_scale
                node_noise = nodes.new(type="ShaderNodeTexNoise")
                node_noise.location = -300,-600
                node_noise.inputs[1].default_value = float(displacement_scale)
                node_math = nodes.new(type="ShaderNodeMath")
                node_math.location = 0,-600
                node_math.operation = 'MULTIPLY'
                node_math.inputs[1].default_value = float(displacement_value)
            except:
                print('Error setting NOISE to floor material')
        # link material nodes (floor)
        links = mat.node_tree.links
        if floorcolorimage:
            link = links.new(node_texture.outputs[0], node_principled.inputs[0])
        if floorbumpimage and not bpy.context.scene.render.use_freestyle:
            link = links.new(node_normalmap.outputs[0], node_normalvector.inputs[1])
            link = links.new(node_normalvector.outputs[0], node_principled.inputs['Normal']) #needs to find the link by name, doesn't connect otherwise??
        link = links.new(node_principled.outputs[0], node_output.inputs[0])
        if node_noise and node_math:
            link = links.new(node_noise.outputs[1], node_math.inputs[0])
            link = links.new(node_math.outputs[0], node_output.inputs[2])
    bpy.context.scene.update()
    return openinblender


##########################################
############# OBJ IMPORTING ##############
##########################################
import bpy, os
class ObjGroup:
    def __init__(self,name):
        self.Name = name
        self.MaterialName = ""
        self.Faces = []
        self.TextureCoords = []
        self.Verts = []

class ObjMaterial:
    def __init__(self,name):
        self.Name = name
        self.Opacity = 1.0
        self.Color = 0.0, 0.0, 0.0
        self.ColorMap = ''
        self.Emission = 0.0
        self.Metallic = 0.0
        self.Roughness = 0.0
        self.NormalMap = ''
        self.Bumpiness = 0.0
        self.Clearcoat = 0.0
        self.ClearcoatRougness = 0.0
        self.CustomData = 'N/A'
        #self.ProceduralDisplacement = 'N/A'
        #self.DisplacementValue = 5.0
        #self.DisplacementScale = 5.0

def stripNonAscii(s):
    ss = ""
    for c in s:
        if ord(c) <128:
            ss += c
    return ss


def setupShaderMaterial(mat,
        col,
        color_map,
        metallic,
        roughness,
        normal_map,
        bumpiness, 
        clearcoat,
        clearcoat_rouhness,
        transparency,
        custom_data):
        node_noise,node_math = None, None
        # get the nodes
        mat.use_nodes = True
        nodes = mat.node_tree.nodes
        for node in nodes:
            nodes.remove(node)
        if not bpy.context.scene.render.use_freestyle:
            # create principled
            node_principled = nodes.new(type="ShaderNodeBsdfPrincipled")
            node_principled.location = 0,0
            node_principled.inputs[4].default_value = metallic#1.0 if metallic > 0 else 0.0 #material is metal or not.. nothing between
            node_principled.inputs[7].default_value = roughness
            node_principled.inputs[12].default_value = clearcoat
            node_principled.inputs[13].default_value = clearcoat_rouhness
            # create output node
            node_output = nodes.new(type='ShaderNodeOutputMaterial')   
            node_output.location = 300,0
            # create texture     node
            if color_map:
                node_texture = nodes.new(type='ShaderNodeTexImage')
                node_texture.location = -600,100
                texture_img = bpy.data.images.load(color_map)
                node_texture.image = texture_img
            else:
                # text color
                if len(col) == 3:
                    ccolor = [0,0,0,1]
                    ccolor[:3] = col
                    col = tuple(ccolor)
                col = [x**2.2 for x in col] # gamma correction
                #if transparency < 1.0:
                #    col = [x/10.0 + 0.9 for x in col]
                node_principled.inputs[0].default_value = col
            if transparency < 1.0: #opacity
                node_principled.inputs[15].default_value = 1.0 #transmission
                col = [x/10.0 + 0.9 for x in col] #make it brighter
            else:
                node_principled.inputs[15].default_value = 0.0 #transmission
            if normal_map:
                node_normalmap = nodes.new(type='ShaderNodeTexImage')
                node_normalmap.location = -600,-300
                normalmap_img = bpy.data.images.load(normal_map)
                node_normalmap.image = normalmap_img
                #
                node_normalvector = nodes.new(type="ShaderNodeNormalMap")
                node_normalvector.location = -300,-300
                node_normalvector.inputs[0].default_value = bumpiness*2
            if custom_data.startswith('NOISE'):
                #displacement_value, displacement_scale
                try:
                    c_type, displacement_scale, displacement_value =  custom_data.split('/')
                    node_noise = nodes.new(type="ShaderNodeTexNoise")
                    node_noise.location = -300,-600
                    node_noise.inputs[1].default_value = float(displacement_scale)
                    node_math = nodes.new(type="ShaderNodeMath")
                    node_math.location = 0,-600
                    node_math.operation = 'MULTIPLY'
                    node_math.inputs[1].default_value = float(displacement_value)
                except:
                    print('Error setting NOISE to "%s" material.' % mat.name)
            # link nodes
            links = mat.node_tree.links
            if color_map:
                link = links.new(node_texture.outputs[0], node_principled.inputs[0])
            if normal_map:
                link = links.new(node_normalmap.outputs[0], node_normalvector.inputs[1])
                link = links.new(node_normalvector.outputs[0], node_principled.inputs['Normal']) #needs to find the link by name, doesn't connect otherwise??
            link = links.new(node_principled.outputs[0], node_output.inputs[0])
            if node_noise and node_math:
                link = links.new(node_noise.outputs[1], node_math.inputs[0])
                link = links.new(node_math.outputs[0], node_output.inputs[2])
        else:
            # create toon shader when freestyle is enabled for cartoonish looks
            node_toon = nodes.new(type="ShaderNodeBsdfToon")
            node_toon.location = 0,0
            # create output node
            node_output = nodes.new(type='ShaderNodeOutputMaterial')   
            node_output.location = 300,0
            # create texture     node
            if color_map:
                node_texture = nodes.new(type='ShaderNodeTexImage')
                node_texture.location = -600,100
                texture_img = bpy.data.images.load(color_map)
                node_texture.image = texture_img
            else:
                # text color_map
                if len(col) == 3:
                    ccolor = [0,0,0,1]
                    ccolor[:3] = col
                    col = tuple(ccolor)
                col = [x**2.2 for x in col] # gamma correction
                node_toon.inputs[0].default_value = col
            node_toon.inputs[1].default_value = 0.7
            node_toon.inputs[2].default_value = 0.0
            # link nodes
            links = mat.node_tree.links
            link = links.new(node_toon.outputs[0], node_output.inputs[0])
            if color_map:
                link = links.new(node_texture.outputs[0], node_toon.inputs[0])


def loadMAT(filename):
    materials = {}
    DIR = os.path.dirname(filename)
    material_name = 'OBJ_import_material'
    current_mat = None
    with open(filename, "r") as mtlfile:
        for line in mtlfile:
            line = stripNonAscii(line.strip())
            if not line or line[0] == '#':
                # skip empty or comment lines
                continue
            vals = line.split()
            line_id = vals[0].lower()
            if line_id == "newmtl":
                material_name = '_'.join(vals[1:]).strip(',.-#')
                if material_name not in materials:
                    materials[material_name] = ObjMaterial(material_name)
                current_mat = materials[material_name]
            elif line_id == 'kd':
                current_mat.Color = (float(vals[1]), float(vals[2]), float(vals[3]))
            elif line_id == 'ks':
                col_specular = (float(vals[1]), float(vals[2]), float(vals[3]))
            elif line_id == 'ke':
                current_mat.Emission = float(vals[1])
            elif line_id == 'd':  # dissolve (transparency)
                current_mat.Opacity = float(vals[1])
            elif line_id == 'bumpiness':  
                current_mat.Bumpiness = float(vals[1])
            elif line_id == 'clearcoat': 
                current_mat.Clearcoat = float(vals[1])
            elif line_id == 'roughness':  
                current_mat.Roughness = float(vals[1])
            elif line_id == 'metallic':  
                current_mat.Metallic = float(vals[1])
            elif line_id == 'map_kd':
                current_mat.ColorMap = os.path.join(str(DIR), vals[1] )
            elif line_id == 'map_bump':
                current_mat.NormalMap = os.path.join(str(DIR), vals[1] )
            elif line_id == 'custom_data':
                current_mat.CustomData = vals[1]
    return materials


def loadOBJ(filename):
    global matData
    groups = {}
    group_name = os.path.splitext(os.path.basename(filename))[0]
    verts = []  
    texts = []
    vindex, tindex = 0, 0
    mtllib = ""
    base_group_name = group_name
    currentline = 0
    prev_verts_count = 0
    with open(filename, "r") as objfile:
        for line in objfile:
            currentline+=1
            line = stripNonAscii(line.strip())
            if not line or line[0] == '#':
                # skip empty or comment lines
                continue
            vals = line.split() 
            if vals[0] == "g":
                group_name = ' '.join(vals[1:])
                if group_name not in groups:
                    groups[group_name] = ObjGroup(group_name) 
                #print('define',group_name, verts)
                groups[group_name].Verts = verts[:]
                vindex += prev_verts_count
                tindex += prev_verts_count
                prev_verts_count = len(verts[:])
                verts = []
                #print(groups[group_name].Verts)
                #print('-------------')
                base_group_name = group_name
            if vals[0] == "usemtl":
                matname = ' '.join(vals[1:]).strip(',.-#')
                #group_name = base_group_name + '_' + matname
                #if group_name not in groups:
                #    groups[group_name] = ObjGroup(group_name) 
                groups[group_name].MaterialName = matname
            if vals[0] == "mtllib":
                mtllib = ' '.join(vals[1:])
            if vals[0] == "v":
                # vertices
                v = tuple(map(float, vals[1:4]))
                verts.append(v)
            if vals[0] == "vt":
                # vertex texture coordinates
                t = tuple(map(float, vals[1:3])) # we use only 2 coordinates (u,v)
                texts.append(t) #e.g. texts.append( (0.175, 0.791) )

            if vals[0] == "f":
                face = []
                text = []
                # FACE
                for f in vals[1:]:  
                    w = f.split("/")  
                    # FACE VERTEX
                    #print(w)
                    try:face.append( int(w[0])-1-vindex ) # obj indexing starts from 1
                    except:
                        print(currentline,'<= on line. Error: vertex indexing', len(verts), [int(w[0])-1-vindex])
                        return
                    # FACE TEXTURE COORDINATE
                    try:     text.append( texts[int(w[1])-1-tindex] )
                    except:  text.append( (0,0) ) # we use only 2 coordinates (u,v)
                # ADD FACE TO THIS GROUP 
                groups[group_name].Faces.append(face)  #(groups ~ dictionary)
                groups[group_name].TextureCoords = tindex
                #groups[group_name].TextureCoords.append(text)  #(groups ~ dictionary)
                #print (groups[group_name].Faces)
                #groups[group_name].TextureCoords.append(text)  #(groups ~ dictionary)
    if verts and group_name in groups:
        # store previous group's verts
        groups[group_name].Verts = verts[:]
        verts = []
    # remove if there are any empty groups
    emptygroups = []
    for g in groups.keys():
        if not groups[g].Faces:
            emptygroups.append(g)
    for e in emptygroups:
        del groups[e]
    return groups, verts, texts


def loadmodel(filename, name, blender_materials):
    basename, extension = os.path.splitext(os.path.basename(filename))
    #items, matData, verts, texts = loadOBJ(filename) 
    items, verts, texts = loadOBJ(filename) 
    #new_empty = bpy.ops.object.add(type='EMPTY', location=inipos[0:3])
    new_empty = bpy.ops.object.add(type='EMPTY', location=[0,0,0])
    empty_object = bpy.context.selected_objects[0]
    empty_object.name = name
    import bmesh
    
    '''
    for itemname in items:
        item = items[itemname]
        print ('------------',itemname)
        print ('verts', item.Verts)
        print ('faces', item.Faces)
    '''
    for itemname in items:
        item = items[itemname]
        mesh_data = bpy.data.meshes.new(itemname)
        #mesh_data.from_pydata(verts, [], item.Faces)
        mesh_data.from_pydata(item.Verts, [], item.Faces)
        mesh_data.update(calc_edges=True)
        obj = bpy.data.objects.new(itemname, mesh_data)
        obj.parent = empty_object
        scene = bpy.context.scene
        scene.objects.link(obj)
        mat = blender_materials.get(item.MaterialName)
        if obj.data.materials:
                obj.data.materials[0] = mat
        else:
                obj.data.materials.append(mat)
        obj.select = True
        for f in obj.data.polygons:
            f.use_smooth = True
        #
        # UV
        #
        bpy.context.scene.objects.active = obj
        bpy.ops.object.mode_set(mode='EDIT')
        bm = bmesh.from_edit_mesh(mesh_data)
        uv_layer = bm.loops.layers.uv.verify()
        bm.faces.layers.tex.verify()
        for f in bm.faces:
            for l in f.loops:
                luv = l[uv_layer]
                luv.uv = tuple(texts[l.vert.index+item.TextureCoords])
                #luv.uv = tuple(item.TextureCoords[l.vert.index][0:2])
        bmesh.update_edit_mesh(mesh_data)
        bpy.ops.object.mode_set(mode='OBJECT')
    return empty_object
##########################################
############# OBJ IMPORTING funcs end ####
##########################################



## MAIN EXECUTION ##
openinblender = do_import_vc()
frameCount = bpy.context.scene.frame_end - bpy.context.scene.frame_start
filepathroot, file_extension = os.path.splitext(bpy.context.scene.render.filepath)
if not openinblender:
    if frameCount == 0:
        bpy.context.scene.frame_set( 0 )
        bpy.ops.render.render( write_still=True )
    else:
        for i in range(frameCount):
            bpy.context.scene.frame_set( i )
            i_filename = "%s%05d%s" % (filepathroot, i, file_extension)
            bpy.context.scene.render.filepath = i_filename
            bpy.ops.render.render( write_still=True )





